
#################################################################################
###               For both general and special cases 
###   Function to calculate     log( gamma(N+1)/gamma(N-n+1) )          
#################################################################################

digam<-function(n_big, n_small)
{ out=0
  if(n_small>0&n_big>=n_small)  out=sum(log(n_big+1-c(1:n_small)))
  out
}


#################################################################################
###             For both general and special cases 
###       minimize   - sum_i  log(1+lambda t_i)  with respect to lambda                      
###   For the given t vector, it returns the Lagrange multiplier lambda        
#################################################################################

el_lagrange<-function(t)
{
  emplik<-function(lam)
  { eps=1e-5;  z=1+lam*t; z1=z[z<eps]/eps
    -sum(log(z[z>eps]))-sum(2*z1-0.5*z1^2+log(eps)-1.5)
   }
  optimize(f=emplik, interval=c(-1000, 1000))
}

#################################################################################
###             For both general and special cases 
###          For the given x, calculate exp(x)/(1+exp(x) )                     
#################################################################################

expit=function(x)
{
plogis(x,0,1)
}


#################################################################################
###                         For general case only             
###   minimize  - h_1(N,alpha) with respect to N for the given alpha    
#################################################################################

glike_part1<-function(n_small, alpha)
{ fgamma <- function(n_big)
{   -digam(n_big, n_small)-(n_big-n_small)*log(alpha)   }
optimize(fgamma, interval=c(n_small, 100*n_small), tol=0.01)
}

#################################################################################
###                         For general case only              
###   minimize -h_23(beta,alpha) with respect to beta for the given alpha 
#################################################################################

glike_part23<-function(x, d,alpha)
{

  dimension=dim(d)
  n_small=dimension[1]
  k=dimension[2]
  xmat=cbind(rep(1,n_small),x)


  fun23<-function(beta)
  {
    dm=length(beta)/k
    nbeta=matrix(beta,nrow=dm,byrow=F)
    xbeta=xmat%*%nbeta
    gbeta=expit(xbeta)
    phi=apply((1-gbeta),1,prod)
    phi=as.numeric(phi)

    result.lagrange=el_lagrange(phi-alpha)

     tt=sum(d*log(gbeta+1e-300)+(1-d)*log(1-gbeta+1e-300))

     lambda=result.lagrange$minimum
     fv =result.lagrange$objective+ tt
   -fv
  }

  gr<-function(beta)
  {
    dm=length(beta)/k
    nbeta=matrix(beta,nrow=dm,byrow=F)
    xbeta=xmat%*%nbeta
    gbeta=expit(xbeta)
    phi=apply((1-gbeta),1,prod)
    phi=as.numeric(phi)
    result.lagrange=el_lagrange(phi-alpha)
    lam= result.lagrange$minimum
    V1=c()
    for(j in 1:k)
    {
     tmp = (lam*alpha-1)/(1+lam*(phi-alpha))*gbeta[,j]
    vj=-(tmp+d[,j])%*%xmat
    V1=c(V1,as.numeric(vj))
   }
V1
}


 dim_q=ncol(xmat)
 initialbeta=rep(0,dim_q*k)
 beta0=initialbeta
 lower=rep(-1000, dim_q*k)
 out=nlminb(beta0, fun23,gradient=gr, lower=lower, upper=-lower)
   list(par=out$par, value=out$objective)
}


#################################################################################
###                          For general case only              
###   Calculate empirical likelihood function for the given N_0  
###   Input                                                               
###       x: a vector  or matrix containing the observed q(x) without intercept       
###       d: a matrix with k columns (k, number of captures), capture history                      
### n_big_0: a given value for the population size,  at which        
###            the empirical log-likelihood function is evaluated           
###                                                                         
###   Output                                                                
###      out$objective:  empirical log-likelihood function at n_big_0   
#################################################################################
gel.null<-function(x,d, n_big_0)
{
  dimension=dim(d)
  n_small=dimension[1]
  k=dimension[2]
  falpha.null<-function(alpha)
   {
    tmp1=-(n_big_0-n_small)*log(alpha)-digam(n_big_0, n_small)
    tmp2=glike_part23(x, d, alpha)$value
    tmp1+tmp2
   }

   eps=1e-6
  out=optimize(falpha.null, interval=c(eps, 1-eps), tol=0.0001)
  -out$objective
}


#################################################################################
###                  For general case only             
###   Implement empirical likelihood method    
###   Input                                                   
###       x: a vector  or matrix containing the observed q(x) without intercept 
###       d: a matrix with k columns (k, number of captures), capture history
###   level: the nominal level for confidence interval
###
###   Output
###      n_hat:     Maximum empirical likelihood estimate of N
###      alpha_hat: Maximum empirical likelihood estimate of alpha
###      beta_hat:  Maximum empirical likelihood estimate of beta
###      prob:      Maximum empirical likelihood estimate of p_i
###      n_small:   Number of observations.
###      lambda:    Final value of the Lagrange multiplier 
###                 in the empirical likelihood
###      ci.elr:    Empirical-likelihood-ratio-based CI
#################################################################################
gel.abun<-function(x,d,level=0.95)
{
  dimension=dim(d)
  n_small=dimension[1]
  k=dimension[2]

falpha<-function(alpha)
   {
      tmp1=glike_part1(n_small, alpha)$objective
      tmp2=glike_part23(x, d, alpha)$value
      tmp1+tmp2
    }

eps=1e-6
out=optimize(falpha, interval=c(eps, 1-eps), tol=0.0001)
like= -out$objective
alpha_est=out$minimum

##############  EL estimator of N  ##############
   out1=glike_part1(n_small, alpha_est)
   n_hat = out1$minimum
#################################################
############## EL estimator of beta  ############
   out23=glike_part23(x,d, alpha_est)
   beta_hat = out23$par
#################################################


 ###########   probability weights of EL  ##############
   beta=beta_hat
   xmat=cbind(rep(1,n_small),x)
   dm=length(beta)/k
   nbeta=matrix(beta,nrow=dm,byrow=F)
   xbeta=xmat%*%nbeta
   gbeta=expit(xbeta)
   phi=apply((1-gbeta),1,prod)
   phi=as.numeric(phi)
   lam = el_lagrange(phi-alpha_est)$minimum

   prob=rep(0, n_small)
   prob = n_small*(1+lam*(phi-alpha_est))
   prob=1/prob

   ###########   ELR based CI  ##############
   rn=function(N)
   {
   like.full=like
   like.null=gel.null(x,d,N)
   2*(like.full-like.null)-qchisq(level,1)
   }

   ntemp=n_hat
   while(rn(ntemp)<=0)
   {
   ntemp=ntemp*2
   }

    ci.upper=uniroot(rn,c(n_hat,ntemp))$root
    if(rn(n_small)<=0) {ci.lower=n_small}
    else
    {
    ci.lower=uniroot(rn,c(n_small,n_hat))$root
    }

    ci=c(ci.lower,ci.upper)

  list(n_hat=n_hat, alpha_est=alpha_est, beta_hat=beta_hat, prob=prob,n_small=n_small, lambda=lam,ci.elr=ci)

}

#################################################################################
###                  For general case only             
###   Implement conditional likelihood method
###   Input
###       x: a vector  or matrix containing the observed q(x) without intercept 
###       d: a matrix with k columns (k, number of captures), capture history
###
###   Output
###      n_tilde:     Maximum conditional likelihood estimate of N
###      beta_tilde:  Maximum conditional likelihood estimate of beta
###      sigma_hat:   Estimate of the asymptotic variance of n_tilde
#################################################################################
gcl.abun<-function(x, d)
{
  dimension=dim(d)
  n_small=dimension[1]
  k=dimension[2]
  xmat=cbind(rep(1,n_small),x)

  fun.c<-function(beta)
   {

    dm=length(beta)/k
    nbeta=matrix(beta,nrow=dm,byrow=F)
    xbeta=xmat%*%nbeta
    gbeta=expit(xbeta)
    phi=apply((1-gbeta),1,prod)
    phi=as.numeric(phi)

    tt=sum(d*log(gbeta+1e-300)+(1-d)*log(1-gbeta+1e-300))


    -sum(tt)+sum(log(1-phi+1e-300))
   }

  dim_q=ncol(xmat)
  beta0=rep(0, k*dim_q)
  out=nlminb(beta0, fun.c)
  beta_tilde=out$par

  V23 = rep(0, dim_q*k)
  V22= matrix(0,dim_q*k,dim_q*k)
  beta=beta_tilde

  nbeta=matrix(beta,nrow=dim_q,byrow=F)
  xbeta=xmat%*%nbeta
  gbeta=expit(xbeta)
  phi=apply((1-gbeta),1,prod)
  phi=as.numeric(phi)
  n_tilde = sum(1/(1-phi+1e-300))
  phistar = sum(1/(1-phi+1e-300)^2)/n_tilde


  for(i in 1:n_small)
   {
       V23i = phi[i]/(1-phi[i]+1e-300)^2 * kronecker(gbeta[i, ], xmat[i, ])
       V23  = V23+V23i
       tmp1 = d[i,] - gbeta[i, ]/(1-phi[i]+1e-300)
       tmp2 = kronecker(tmp1, xmat[i, ])
       V22i = tmp2%*%t(tmp2)
       V22=V22+V22i
      }
V23=V23/n_tilde
V22=-V22/n_tilde
V22.inv=solve(V22-diag(1e-300,dim_q*k,dim_q*k))
tmp = t(V23)%*%V22.inv%*%V23
tmp=as.numeric(tmp)
sigma_hat =  phistar - 1-tmp



list(n_tilde=n_tilde, beta_tilde= beta,sigma_hat =sigma_hat)
}

#################################################################################
###                  Main function for general case, i.e. M_th model
#################################################################################
###   Implement the empirical likelihood and conditional likelihood methods
###   Input
###       x: a vector  or matrix containing the observed q(x) without intercept 
###       d: a matrix with k columns (k, number of captures), capture history
###   level: the nominal level for confidence interval
###
###   Output
###     stat:     A vector contains n_small, N_hat, N_tilde, 
###                alpha_hat, sigma_hat, lambda
###
###     CI  :     Four types of CI for N: 
###                 empirical likelihood ratio based CI
###                 and three Wald-type CIS 
###
### beta_est:     Maximum empirical likelihood estimate of beta
###               and maximum conditional likelihood estimate of beta
#################################################################################

gabun=function(x,d,level=0.95)
{
x=as.matrix(x)
minx=apply(x,2,min)
maxx=apply(x,2,max)
rx=maxx-minx
mrx=matrix(rx,nrow=nrow(x),byrow=T,ncol=ncol(x))
newx=x/mrx
dm=ncol(x)+1
out.el=gel.abun(newx,d,level=level)
out.cl=gcl.abun(newx,d)
beta_hat=out.el$beta_hat
beta_tilde=out.cl$beta_tilde

k=ncol(d)
for(i in 1:k)
{
index=(2:dm)+(i-1)*dm
beta_hat[index]=beta_hat[index]/rx
beta_tilde[index]=beta_tilde[index]/rx
}

out.el$beta_hat=beta_hat
out.cl$beta_tilde=beta_tilde

interval_EL=out.el$ci.elr

interval_CL=matrix(rep(0, 6), ncol=2)

n_small=nrow(d)
sigma_hat=out.cl$sigma_hat
n_tilde = out.cl$n_tilde
tmp=qnorm(0.5+level/2,0,1)*sqrt(sigma_hat*n_tilde)
tmp1=tmp/n_tilde
interval_CL[1,1]= n_tilde - tmp
interval_CL[1,2]= n_tilde + tmp
interval_CL[2,1]= n_tilde*exp(-tmp1)
interval_CL[2,2]= n_tilde*exp(tmp1)

c1= 1+n_tilde*sigma_hat/(n_tilde - n_small+1e-300)^2
sdc1=sqrt(log(c1))
interval_CL[3,1]= n_small + exp(log(n_tilde-n_small+1e-300)-qnorm(0.5+level/2,0,1)*sdc1)
interval_CL[3,2]= n_small + exp(log(n_tilde-n_small+1e-300)+qnorm(0.5+level/2,0,1)*sdc1)
interval=rbind(interval_EL, interval_CL)

rownames(interval)<-c("I1(EL)", "I2(CL1)", "I3(CL2)", "I4(Chao)")
colnames(interval)<-c("Lower_Limit", "Upper_Limit")
interval=round(interval)

stat=c(out.el$n_small, round(out.el$n_hat), round(out.cl$n_tilde),
       round(out.el$alpha_est,4), round(out.cl$sigma_hat,4), round(out.el$lambda,4))

stat=matrix(stat, nrow=1)

colnames(stat)=c('n_small', 'n_hat', 'n_tilde', 'alpha_hat',  'sigma_hat', 'lambda' )

beta_est = rbind(out.el$beta_hat, out.cl$beta_tilde)
rownames(beta_est) <- c("EL", "CL")
list(stat=stat, CI=interval, beta_est=beta_est)
	
}



#################################################################################
###                         For special case only             
###   minimize  - h_1(N,alpha) with respect to N for the given alpha    
#################################################################################
slike_part1<-function(n_small, alpha)
{ fgamma <- function(n_big)
{   -digam(n_big, n_small)-(n_big-n_small)*log(alpha)   }
optimize(fgamma, interval=c(n_small, 100*n_small), tol=0.01)
}

#################################################################################
###                         for special case only              
###   minimize -h_23(beta,alpha) with respect to beta for the given alpha 
#################################################################################
slike_part23<-function(x, d, k, alpha)
{

  n_small = length(d)
  xmat=cbind(rep(1,n_small),x)


  fun23<-function(beta)
  {
    xbeta=xmat%*%beta
    xbeta=as.numeric(xbeta)
    gbeta=expit(xbeta)
    phi=(1-gbeta)^k
    result.lagrange=el_lagrange(phi-alpha)

    tt=sum(d*log(gbeta+1e-300)+(k-d)*log(1-gbeta+1e-300))

    lambda=result.lagrange$minimum
    fv =result.lagrange$objective+ tt
    -fv
  }

 gr<-function(beta)
  {
    xbeta=xmat%*%beta
    xbeta=as.numeric(xbeta)
    gbeta=expit(xbeta)
    phi=(1-gbeta)^k
    result.lagrange=el_lagrange(phi-alpha)
    lam= result.lagrange$minimum
    tmp = (lam*alpha-1)/(1+lam*(phi-alpha))*k*gbeta
    V1=-t(xmat)%*%(tmp+d)
    as.numeric(V1)
  }

  dim_q=ncol(xmat)
  initialbeta=rep(0,dim_q)
  beta0=initialbeta
  lower=rep(-1000, dim_q)
  out=nlminb(beta0, fun23,gradient=gr, lower=lower, upper=-lower)
  list(par=out$par, value=out$objective)
}


#################################################################################
###                          For special case only              
###   Calculate empirical likelihood function for the given N_0  
###   Input                                                               
###       x: a vector or matrix containing the observed q(x) without intercept 
###       d: a vector, numbers of times that individuals 
###                   are captured in k captures                 
###       k: the number of captures.                                       
### n_big_0: a given value for the population size,  at which        
###           the empirical log-likelihood function is evaluated           
###                                                                         
###   Output                                                                
###      out$objective:     empirical log-likelihood function at n_big_0   
#################################################################################
sel.null<-function(x,d, k, n_big_0)
{
   n_small = length(d)
   falpha.null<-function(alpha)
   {
     tmp1=-(n_big_0-n_small)*log(alpha)-digam(n_big_0, n_small)
      tmp2=slike_part23(x, d, k, alpha)$value
      tmp1+tmp2
   }

   eps=1e-6
  out=optimize(falpha.null, interval=c(eps, 1-eps), tol=0.0001)
  -out$objective
}


#################################################################################
###                  For special case only             
###   Implement empirical likelihood method    
###   Input                                                   
###       x: a vector  or matrix containing the observed q(x) without intercept       
###       d: a vector, numbers of times that individuals 
###                   are captured in k captures                 
###       k: the number of captures.                     
###  level:  the nominal level for confidence interval
###
###   Output
###      n_hat:     Maximum empirical likelihood estimate of N
###      alpha_hat: Maximum empirical likelihood estimate of alpha
###      beta_hat:  Maximum empirical likelihood estimate of beta
###      prob:      Maximum empirical likelihood estimate of p_i
###      n_small:   Number of observations
###      lambda:    Final value of the Lagrange multiplier 
###                 in the empirical likelihood
###      ci.elr:    Empirical-likelihood-ratio-based CI
#################################################################################
sel.abun<-function(x,d, k,level=0.95)
{
  n_small=length(d)
  falpha<-function(alpha)
   {
      tmp1=slike_part1(n_small, alpha)$objective
      tmp2=slike_part23(x, d, k, alpha)$value
      tmp1+tmp2
    }

   eps=1e-6
   out=optimize(falpha, interval=c(eps, 1-eps), tol=0.0001)
   like= -out$objective
   alpha_est=out$minimum

##############  EL estimator of N  ##############
   out1=slike_part1(n_small, alpha_est)
   n_hat = out1$minimum
#################################################
############## EL estimator of beta  ############
   out23=slike_part23(x,d, k, alpha_est)
   beta_hat = out23$par
#################################################


 ###########   probability weights of EL  ##############
   beta=beta_hat
   xmat=cbind(rep(1,n_small),x)
   xbeta=xmat%*%beta
   xbeta=as.numeric(xbeta)
   gbeta=expit(xbeta)
   phi=(1-gbeta)^k
   lam = el_lagrange(phi-alpha_est)$minimum

   prob=rep(0, n_small)
   prob = n_small*(1+lam*(phi-alpha_est))
   prob=1/prob

   ###########   ELR based CI  ##############
   rn=function(N)
   {
     like.full=like
     like.null=sel.null(x,d,k,N)
     2*(like.full-like.null)-qchisq(level,1)
   }

   ntemp=n_hat
   while(rn(ntemp)<=0)
   {
     ntemp=ntemp*2
   }

   ci.upper=uniroot(rn,c(n_hat,ntemp))$root
   if(rn(n_small)<=0) {ci.lower=n_small}
   else{
     ci.lower=uniroot(rn,c(n_small,n_hat))$root
   }

   ci=c(ci.lower,ci.upper)

   list(n_hat=n_hat, alpha_est=alpha_est, beta_hat=beta_hat, prob=prob,n_small=n_small, lambda=lam,ci.elr=ci)
}

#################################################################################
###                  For special case only             
###   Implement conditional likelihood method
###   Input
###       x: a vector  or matrix containing the observed q(x) without intercept        
###       d: a vector, numbers of times that individuals 
###                   are captured in k captures               
###       k: the number of captures. 
###
###   Output
###      n_tilde:     Maximum conditional likelihood estimate of N
###      beta_tilde:  Maximum conditional likelihood estimate of beta
###      sigma_hat:   Estimate of the asymptotic variance of n_tilde
#################################################################################
scl.abun<-function(x, d, k)
{
  n_small=length(d)
  xmat=cbind(rep(1,n_small),x)

  fun.c<-function(beta)
   {
     xbeta=xmat%*%beta
     xbeta=as.numeric(xbeta)
     gbeta=expit(xbeta)
     phi=(1-gbeta)^k
     tt=sum(d*log(gbeta+1e-300)+(k-d)*log(1-gbeta+1e-300))
    -sum(tt)+sum(log(1-phi+1e-300))
   }

  dim_q=ncol(xmat)
  beta0=rep(0, dim_q)
  out=nlminb(beta0, fun.c)
  beta_tilde=out$par

  V23 = rep(0, dim_q)
  V22= matrix(0,dim_q,dim_q)
  beta=beta_tilde
  xbeta=xmat%*%beta
  xbeta=as.numeric(xbeta)
  gbeta=expit(xbeta)
  phi=(1-gbeta)^k
  n_tilde = sum(1/(1-phi+1e-300))
  phistar=sum(1/(1-phi+1e-300)^2)/n_tilde

  V23.coef=(phi/(1-phi+1e-300)^2)*k*gbeta
  V22.coef=(d-k*gbeta/(1-phi+1e-300))^2
  V22.part2=xmat

for(i in 1:n_small)
   {

       V23=V23+V23.coef[i]*xmat[i,]
       qxi=matrix(xmat[i,],ncol=1)

       V22=V22+V22.coef[i]*qxi%*%t(qxi)

      }

   V23=V23/n_tilde;
   V22=-V22/n_tilde
   V23=matrix(V23,ncol=1)
   V32=t(V23)
   V22.inv=solve(V22-diag(1e-300,dim_q,dim_q));
   tmp = V32%*%V22.inv%*%V23
   tmp=as.numeric(tmp)

   sigma_hat =  phistar - 1 -tmp

list(n_tilde=n_tilde, beta_tilde= beta,  sigma_hat =sigma_hat )
}

#################################################################################
###                  Main function for special case, i.e. M_h model
#################################################################################
###   Implement the empirical likelihood and conditional likelihood methods
###   Input
###       x: a vector  or matrix containing the observed q(x) without intercept        
###       d: a vector, numbers of times that individuals 
###                   are captured in k captures                 
###       k: the number of captures. 
###   level: the nominal level for confidence interval
###
###   Output
###     stat:     A vector contains n_small, N_hat, N_tilde, 
###                alpha_hat, sigma_hat, lambda
###
###     CI  :     Four types of CI for N: 
###                 empirical likelihood ratio based CI
###                 and three Wald-type CIS 
###
### beta_est:     Maximum empirical likelihood estimate of beta
###               and maximum conditional likelihood estimate of beta
#################################################################################


sabun=function(x,d,k,level=0.95)
{
x=as.matrix(x)
minx=apply(x,2,min)
maxx=apply(x,2,max)
rx=maxx-minx
mrx=matrix(rx,nrow=nrow(x),byrow=T,ncol=ncol(x))
newx=x/mrx
out.el=sel.abun(newx,d,k,level=level)
out.cl=scl.abun(newx,d,k)
out.el$beta_hat[-1]=out.el$beta_hat[-1]/rx
out.cl$beta_tilde[-1]=out.cl$beta_tilde[-1]/rx
interval_EL=out.el$ci.elr
interval_CL=matrix(rep(0, 6), ncol=2)


n_small=length(d)
sigma_hat=out.cl$sigma_hat
n_tilde = out.cl$n_tilde
tmp=qnorm(0.5+level/2,0,1)*sqrt(sigma_hat*n_tilde)
tmp1=tmp/n_tilde
interval_CL[1,1]= n_tilde - tmp
interval_CL[1,2]= n_tilde + tmp
interval_CL[2,1]= n_tilde*exp(-tmp1)
interval_CL[2,2]= n_tilde*exp(tmp1)

c1= 1+n_tilde*sigma_hat/(n_tilde - n_small+1e-300)^2
sdc1=sqrt(log(c1))
interval_CL[3,1]= n_small + exp(log(n_tilde-n_small+1e-300)-qnorm(0.5+level/2,0,1)*sdc1)
interval_CL[3,2]= n_small + exp(log(n_tilde-n_small+1e-300)+qnorm(0.5+level/2,0,1)*sdc1)
interval=rbind(interval_EL, interval_CL)
rownames(interval)<-c("I1(EL)", "I2(CL1)", "I3(CL2)", "I4(Chao)")
colnames(interval)<-c("Lower_Limit", "Upper_Limit")
interval=round(interval)



stat=c(out.el$n_small, round(out.el$n_hat), round(out.cl$n_tilde),
       round(out.el$alpha_est,4), round(out.cl$sigma_hat,4), round(out.el$lambda,4))

stat=matrix(stat, nrow=1)

colnames(stat)=c('n_small', 'n_hat', 'n_tilde', 'alpha_hat',  'sigma_hat', 'lambda' )

beta_est = rbind(out.el$beta_hat, out.cl$beta_tilde)
rownames(beta_est) <- c("EL", "CL")
list(stat=stat, CI=interval, beta_est=beta_est)
	
}


